//+--------------------------------------------------------------------------------------------+
//|                                                                                NewMoon.mq4 |
//|                                                             Copyright  2012, Serega Lykov |
//|                                                                 http://mtexperts.narod.ru/ |
//+--------------------------------------------------------------------------------------------+

#property copyright "Copyright  2012, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

//---- property of indicator ------------------------------------------------------------------+
#property indicator_chart_window
#property indicator_buffers 0

//---- constants ------------------------------------------------------------------------------+
#define ObjectsSignature_NM           "NewMoon "
#define ObjectsSignature_FM           "FullMoon "

//---- external parameters --------------------------------------------------------------------+
extern int    GMT_Offset                      = 0;
extern bool   NewMoon_Show                    = true;
extern color  NewMoon_Line_Color              = Red;
extern int    NewMoon_Line_Style              = STYLE_DOT;
extern int    NewMoon_Line_Width              = 0;
extern bool   FullMoon_Show                   = false;
extern color  FullMoon_Line_Color             = Yellow;
extern int    FullMoon_Line_Style             = STYLE_DOT;
extern int    FullMoon_Line_Width             = 0;

//---- global variables -----------------------------------------------------------------------+
static datetime bartime;
static int      gmt_offset_seconds;
static int      last_year;
static int      last_month;
static double   Koef_Century[10] = {1.5, 6.8, 11.2, 16.5, 21.8, 27.2, 2.0, 7.3, 12.7, 18.0};  // from 1800 to 2799 years
static double   Koef_Year[10][10] = {0.0,  9.8,  18.6, 28.4, 7.6,  17.4, 26.2, 6.5,  15.3, 25.1,
                                     18.9, 28.7, 8.0,  17.7, 26.5, 8.8,  15.6, 25.4, 4.6,  14.4,
                                     8.3,  17.1, 26.9, 6.1,  15.9, 24.7, 5.0,  13.8, 23.5, 2.8,
                                     27.2, 6.4,  16.2, 25.0, 5.3,  14.1, 23.9, 3.1,  12.9, 21.7,
                                     15.5, 25.3, 4.6,  14.4, 23.2, 3.4,  12.2, 22.0, 1.8,  11.1,
                                     4.9,  14.7, 23.5, 3.7,  12.5, 22.3, 1.6,  11.4, 20.2, 0.5,
                                     23.8, 3.1,  12.8, 21.6, 1.9,  10.7, 20.5, 29.3, 9.5,  18.3,
                                     13.2, 22.0, 2.2,  11.0, 20.8, 0.1,  9.9,  18.7, 28.4, 7.7,
                                     1.5,  11.3, 20.1, 0.4,  9.2,  19.0, 27.8, 8.0,  16.8, 26.6,
                                     20.4, 0.7,  9.5,  19.3, 28.1, 8.3,  17.1, 26.9, 6.2,  16.0};
static double   Koef_Month[12] = {24.0, 22.5, 24.1, 22.6, 22.1, 20.7, 20.2, 18.7, 17.2, 16.8, 15.3, 14.8};
static int      DaysInMonth[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

//---------------------------------------------------------------------------------------------+
//---- initialization of indicator ------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int init()
  {
   gmt_offset_seconds = GMT_Offset * 3600;
   //---- set a "short" name of the indicator -------------------------------------------------+
   IndicatorShortName("NewMoon");
   //---- finish of initialization ------------------------------------------------------------+
   return(0);
   Print("---- Programming by Serega Lykov, http://mtexperts.narod.ru/ ----");
  }

//---------------------------------------------------------------------------------------------+
//---- deinitialization of indicator ----------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int deinit()
  {
   if(NewMoon_Show)  DeleteObjects(ObjectsSignature_NM);
   if(FullMoon_Show) DeleteObjects(ObjectsSignature_FM);
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- start ----------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int start()
  {
   if(bartime == Time[0]) return(0);
   if(bartime == 0) int start_barshift = Bars - 1;
   else start_barshift = 0;
   bartime = Time[0];
   for(int i=start_barshift; i>=0; i--)
     {
      int year  = TimeYear (Time[i]);
      int month = TimeMonth(Time[i]);
      if(year == last_year && month == last_month) continue;
      int index_century = year / 100 - 18;
      int year_in_century = year % 100;
      int index_year_ten = year_in_century / 10;
      int index_year_one = year_in_century % 10;
      int index_month = month - 1;
      if(month > 2) double corr_month = 0.0;
      else if((year % 4) != 0) corr_month = 0.0;
           else corr_month = 1.0;
      double day_of_new_moon = Koef_Century[index_century] + Koef_Year[index_year_one][index_year_ten] + Koef_Month[index_month] + corr_month;
      if(month != 2) corr_month = 0.0;
        {
         if((year % 4) != 0) corr_month = 0.0;
         else corr_month = 1.0;
        }
      while(day_of_new_moon > 1)
        {
         if(day_of_new_moon <= (DaysInMonth[index_month] + corr_month))
           {
            datetime date_of_new_moon = Time[i] - TimeDay(Time[i])*86400 - TimeHour(Time[i])*3600 - TimeMinute(Time[i])*60 - TimeSeconds(Time[i]) + day_of_new_moon*86400 - gmt_offset_seconds;
            if(NewMoon_Show) CreateObjectVLine(StringConcatenate(ObjectsSignature_NM,TimeToStr(date_of_new_moon,TIME_DATE|TIME_MINUTES|TIME_SECONDS)),date_of_new_moon,NewMoon_Line_Style,NewMoon_Line_Width,NewMoon_Line_Color,true);
            if(FullMoon_Show)
              {
               datetime date_of_full_moon = date_of_new_moon + 1274400;
               CreateObjectVLine(StringConcatenate(ObjectsSignature_FM,TimeToStr(date_of_full_moon,TIME_DATE|TIME_MINUTES|TIME_SECONDS)),date_of_full_moon,FullMoon_Line_Style,FullMoon_Line_Width,FullMoon_Line_Color,true);
              }
           }
         day_of_new_moon -= 29.5;
        }
      last_year  = year;
      last_month = month;
     }
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- CreateObjectVLine ----------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void CreateObjectVLine(string obj_name, datetime obj_time1, int obj_style, int obj_width, color obj_color, bool obj_back = false, int obj_window = 0, string obj_text= "")
  {
   if(ObjectFind(obj_name) < 0) ObjectCreate(obj_name,OBJ_VLINE,obj_window,obj_time1,Close[0]);
   else if(ObjectGet(obj_name,OBJPROP_TIME1) != obj_time1) ObjectMove(obj_name,0,obj_time1,Close[0]);
   ObjectSet(obj_name,OBJPROP_COLOR,obj_color); 
   ObjectSet(obj_name,OBJPROP_STYLE,obj_style);
   ObjectSet(obj_name,OBJPROP_WIDTH,obj_width);
   ObjectSet(obj_name,OBJPROP_BACK, obj_back);
   if(StringLen(obj_text) > 0) ObjectSetText(obj_name,obj_text,10); 
  }

//---------------------------------------------------------------------------------------------+
//---- DeleteObjects --------------------------------------------------------------------------+
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+
//---------------------------------------------------------------------------------------------+
void DeleteObjects(string obj_name_signature, int obj_type=-1)
  {
   int obj_total = ObjectsTotal() - 1;
   for(int i=obj_total; i>=0; i--)
     {
      string obj_name = ObjectName(i);
      if(obj_type >= 0) if(ObjectType(obj_name) != obj_type) continue;
      if(StringFind(obj_name,obj_name_signature,0) != 0) continue;
      ObjectDelete(obj_name);
     }
  }
  
//-------------------------------- programming by Serega Lykov, http://mtexperts.narod.ru/ ----+